/*
 * Copyright 2011 Chris de Vreeze
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package eu.cdevreeze.springjdbc

import java.{ util => jutil }
import java.sql.{ Connection, Statement, PreparedStatement }
import scala.collection.{ immutable, mutable }
import scala.collection.JavaConverters._
import scala.reflect.ClassManifest
import org.springframework.{ jdbc => springjdbc }
import org.springframework.jdbc.core.{ JdbcOperations => _, JdbcTemplate => _, _ }

/**
 * Scala `JdbcTemplate`, wrapping a Spring `JdbcTemplate`. Unlike the wrapped Spring counterpart, immutable Scala collections
 * are used instead of Java collections (and Options instead of null).
 *
 * Other than that, the API closely resembles the wrapped API. As a consequence, this wrapper API also heavily uses overloaded
 * methods. This implies that the Spring callback interfaces are used as parameters instead of Scala functions. Fortunately,
 * using factory methods in `JdbcTemplateUtils`, Scala functions can easily be converted to Spring JDBC callback interfaces.
 *
 * For the methods named `execute` in the wrapped Spring counterpart, if needed, use that wrapped Spring counterpart.
 * The same holds for some of the "callback-based" `query` methods.
 *
 * This API is more regular in the passed query arguments than the wrapped Spring API.
 * For example, in `query` methods, typically the first parameter is the SQL string, the second parameter concerns query parameters,
 * and the third parameter, if any, concerns result set handling. Unlike the wrapped Spring counterpart, a parameter concerning
 * result set handling never comes before parameters concerning query parameters.
 *
 * @author Chris de Vreeze
 */
final class JdbcTemplate(val wrappedObject: springjdbc.core.JdbcTemplate) extends JdbcOperations {

  // Batch updates

  def batchUpdate(sqlSeq: immutable.Seq[String]): immutable.IndexedSeq[Int] = {
    wrappedObject.batchUpdate(sqlSeq.toArray).toIndexedSeq
  }

  def batchUpdate(sql: String, pss: BatchPreparedStatementSetter): immutable.IndexedSeq[Int] = {
    wrappedObject.batchUpdate(sql, pss).toIndexedSeq
  }

  // Using typed args

  def query[A](sql: String, typedArgs: TypedArgs, rse: ResultSetExtractor[A]): A = {
    wrappedObject.query(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray, rse)
  }

  def query[A](sql: String, typedArgs: TypedArgs, rch: RowCallbackHandler): Unit = {
    wrappedObject.query(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray, rch)
  }

  def query[A](sql: String, typedArgs: TypedArgs, rowMapper: RowMapper[A]): immutable.IndexedSeq[A] = {
    wrappedObject.query(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray, rowMapper).asScala.toIndexedSeq
  }

  // Using args without SQL types

  def query[A](sql: String, args: Args, rse: ResultSetExtractor[A]): A = {
    wrappedObject.query(sql, args.toArray, rse)
  }

  def query[A](sql: String, args: Args, rch: RowCallbackHandler): Unit = {
    wrappedObject.query(sql, args.toArray, rch)
  }

  def query[A](sql: String, args: Args, rowMapper: RowMapper[A]): immutable.IndexedSeq[A] = {
    wrappedObject.query(sql, args.toArray, rowMapper).asScala.toIndexedSeq
  }

  // Using PreparedStatementSetters

  def query[A](sql: String, pss: PreparedStatementSetter, rse: ResultSetExtractor[A]): A = {
    wrappedObject.query(sql, pss, rse)
  }

  def query[A](sql: String, pss: PreparedStatementSetter, rch: RowCallbackHandler): Unit = {
    wrappedObject.query(sql, pss, rch)
  }

  def query[A](sql: String, pss: PreparedStatementSetter, rowMapper: RowMapper[A]): immutable.IndexedSeq[A] = {
    wrappedObject.query(sql, pss, rowMapper).asScala.toIndexedSeq
  }

  // Query for Int

  def queryForInt(sql: String, typedArgs: TypedArgs): Int = {
    wrappedObject.queryForInt(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray)
  }

  def queryForInt(sql: String, args: Args): Int = {
    wrappedObject.queryForInt(sql, args.toArray: _*)
  }

  // Query for Long

  def queryForLong(sql: String, typedArgs: TypedArgs): Long = {
    wrappedObject.queryForLong(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray)
  }

  def queryForLong(sql: String, args: Args): Long = {
    wrappedObject.queryForLong(sql, args.toArray: _*)
  }

  // Query for Map (record)

  def queryForMap(sql: String, typedArgs: TypedArgs): Map[String, AnyRef] = {
    val javaResult = wrappedObject.queryForMap(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray)
    javaResult.asScala.toMap
  }

  def queryForMap(sql: String, args: Args): Map[String, AnyRef] = {
    val javaResult = wrappedObject.queryForMap(sql, args.toArray: _*)
    javaResult.asScala.toMap
  }

  // Query for Seq (sequence of records)

  def queryForSeq[A](sql: String, typedArgs: TypedArgs, cls: Class[A]): immutable.IndexedSeq[A] = {
    wrappedObject.queryForList(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray, cls).asScala.toIndexedSeq
  }

  def queryForSeq[A](sql: String, typedArgs: TypedArgs): immutable.IndexedSeq[Map[String, AnyRef]] = {
    val javaResult = wrappedObject.queryForList(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray)
    javaResult.asScala.toIndexedSeq map { (row: jutil.Map[String, AnyRef]) => row.asScala.toMap }
  }

  def queryForSeq[A](sql: String, args: Args, cls: Class[A]): immutable.IndexedSeq[A] = {
    wrappedObject.queryForList(sql, args.toArray, cls).asScala.toIndexedSeq
  }

  def queryForSeq[A](sql: String, args: Args): immutable.IndexedSeq[Map[String, AnyRef]] = {
    val javaResult = wrappedObject.queryForList(sql, args.toArray: _*)
    javaResult.asScala.toIndexedSeq map { (row: jutil.Map[String, AnyRef]) => row.asScala.toMap }
  }

  // Query for Object

  def queryForObject[A](sql: String, typedArgs: TypedArgs, cls: Class[A]): A = {
    wrappedObject.queryForObject(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray, cls)
  }

  def queryForObject[A](sql: String, args: Args, cls: Class[A]): A = {
    wrappedObject.queryForObject(sql, args.toArray, cls)
  }

  def queryForObject[A](sql: String, typedArgs: TypedArgs, rowMapper: RowMapper[A]): A = {
    wrappedObject.queryForObject(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray, rowMapper)
  }

  def queryForObject[A](sql: String, args: Args, rowMapper: RowMapper[A]): A = {
    wrappedObject.queryForObject(sql, args.toArray, rowMapper)
  }

  // Updates

  def update(sql: String, args: Args): Int = {
    wrappedObject.update(sql, args.toArray: _*)
  }

  def update(sql: String, typedArgs: TypedArgs): Int = {
    wrappedObject.update(sql, typedArgs.toArgArray, typedArgs.toArgTypeArray)
  }

  def update(sql: String, pss: PreparedStatementSetter): Int = {
    wrappedObject.update(sql, pss)
  }
}
