/*
 * Copyright 2011 Chris de Vreeze
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package eu.cdevreeze.springjdbc
package namedparam

import java.{ util => jutil }
import java.sql.{ Connection, Statement, PreparedStatement }
import scala.collection.{ immutable, mutable }
import scala.collection.JavaConverters._
import scala.reflect.ClassManifest
import org.springframework.{ jdbc => springjdbc }
import org.springframework.jdbc.core.{ JdbcOperations => _, JdbcTemplate => _, _ }
import org.springframework.jdbc.core.namedparam._

/**
 * Scala `NamedParamJdbcTemplate`, wrapping a Spring `NamedParameterJdbcTemplate`. Unlike the wrapped Spring counterpart, immutable Scala collections
 * are used instead of Java collections (and Options instead of null).
 *
 * Other than that, the API closely resembles the wrapped API. As a consequence, this wrapper API also heavily uses overloaded
 * methods. This implies that the Spring callback interfaces are used as parameters instead of Scala functions. Fortunately,
 * using factory methods in `JdbcTemplateUtils`, Scala functions can easily be converted to Spring JDBC callback interfaces.
 *
 * For the methods named `execute` in the wrapped Spring counterpart, if needed, use that wrapped Spring counterpart.
 *
 * @author Chris de Vreeze
 */
final class NamedParamJdbcTemplate(val wrappedObject: springjdbc.core.namedparam.NamedParameterJdbcTemplate) extends NamedParamJdbcOperations {

  // TODO Batch updates

  // Using typed args

  def query[A](sql: String, typedArgs: NamedTypedArgs, rse: ResultSetExtractor[A]): A = {
    wrappedObject.query(sql, typedArgs.toSqlParameterSource, rse)
  }

  def query[A](sql: String, typedArgs: NamedTypedArgs, rch: RowCallbackHandler): Unit = {
    wrappedObject.query(sql, typedArgs.toSqlParameterSource, rch)
  }

  def query[A](sql: String, typedArgs: NamedTypedArgs, rowMapper: RowMapper[A]): immutable.IndexedSeq[A] = {
    wrappedObject.query(sql, typedArgs.toSqlParameterSource, rowMapper).asScala.toIndexedSeq
  }

  // Using args without SQL types

  def query[A](sql: String, args: NamedArgs, rse: ResultSetExtractor[A]): A = {
    wrappedObject.query(sql, args.toJavaMap, rse)
  }

  def query[A](sql: String, args: NamedArgs, rch: RowCallbackHandler): Unit = {
    wrappedObject.query(sql, args.toJavaMap, rch)
  }

  def query[A](sql: String, args: NamedArgs, rowMapper: RowMapper[A]): immutable.IndexedSeq[A] = {
    wrappedObject.query(sql, args.toJavaMap, rowMapper).asScala.toIndexedSeq
  }

  // Query for Int

  def queryForInt(sql: String, typedArgs: NamedTypedArgs): Int = {
    wrappedObject.queryForInt(sql, typedArgs.toSqlParameterSource)
  }

  def queryForInt(sql: String, args: NamedArgs): Int = {
    wrappedObject.queryForInt(sql, args.toJavaMap)
  }

  // Query for Long

  def queryForLong(sql: String, typedArgs: NamedTypedArgs): Long = {
    wrappedObject.queryForLong(sql, typedArgs.toSqlParameterSource)
  }

  def queryForLong(sql: String, args: NamedArgs): Long = {
    wrappedObject.queryForLong(sql, args.toJavaMap)
  }

  // Query for Map (record)

  def queryForMap(sql: String, typedArgs: NamedTypedArgs): Map[String, AnyRef] = {
    val javaResult = wrappedObject.queryForMap(sql, typedArgs.toSqlParameterSource)
    javaResult.asScala.toMap
  }

  def queryForMap(sql: String, args: NamedArgs): Map[String, AnyRef] = {
    val javaResult = wrappedObject.queryForMap(sql, args.toJavaMap)
    javaResult.asScala.toMap
  }

  // Query for Seq (sequence of records)

  def queryForSeq[A](sql: String, typedArgs: NamedTypedArgs, cls: Class[A]): immutable.IndexedSeq[A] = {
    wrappedObject.queryForList(sql, typedArgs.toSqlParameterSource, cls).asScala.toIndexedSeq
  }

  def queryForSeq[A](sql: String, typedArgs: NamedTypedArgs): immutable.IndexedSeq[Map[String, AnyRef]] = {
    val javaResult = wrappedObject.queryForList(sql, typedArgs.toSqlParameterSource)
    javaResult.asScala.toIndexedSeq map { (row: jutil.Map[String, AnyRef]) => row.asScala.toMap }
  }

  def queryForSeq[A](sql: String, args: NamedArgs, cls: Class[A]): immutable.IndexedSeq[A] = {
    wrappedObject.queryForList(sql, args.toJavaMap, cls).asScala.toIndexedSeq
  }

  def queryForSeq[A](sql: String, args: NamedArgs): immutable.IndexedSeq[Map[String, AnyRef]] = {
    val javaResult = wrappedObject.queryForList(sql, args.toJavaMap)
    javaResult.asScala.toIndexedSeq map { (row: jutil.Map[String, AnyRef]) => row.asScala.toMap }
  }

  // Query for Object

  def queryForObject[A](sql: String, typedArgs: NamedTypedArgs, cls: Class[A]): A = {
    wrappedObject.queryForObject(sql, typedArgs.toSqlParameterSource, cls)
  }

  def queryForObject[A](sql: String, args: NamedArgs, cls: Class[A]): A = {
    wrappedObject.queryForObject(sql, args.toJavaMap, cls)
  }

  def queryForObject[A](sql: String, typedArgs: NamedTypedArgs, rowMapper: RowMapper[A]): A = {
    wrappedObject.queryForObject(sql, typedArgs.toSqlParameterSource, rowMapper)
  }

  def queryForObject[A](sql: String, args: NamedArgs, rowMapper: RowMapper[A]): A = {
    wrappedObject.queryForObject(sql, args.toJavaMap, rowMapper)
  }

  // Updates

  def update(sql: String, args: NamedArgs): Int = {
    wrappedObject.update(sql, args.toJavaMap)
  }

  def update(sql: String, typedArgs: NamedTypedArgs): Int = {
    wrappedObject.update(sql, typedArgs.toSqlParameterSource)
  }
}
